package com.softwareag.example.dirservice.crowd;

public class ConnectionConfig {
	private String connectionUrl;
	private String username;
	private String password;
	
	private boolean supportNestedGroups = false;
	
	private boolean useNestedQueries = false;
	
	public ConnectionConfig() {}
	
	public ConnectionConfig(String data) {
		if (data != null) {
			String[] parts = data.split("##");
			if (parts.length > 2) {
				connectionUrl = parts[0];
				username = parts[1];
				password = parts[2];
				if (parts.length > 3) {
					supportNestedGroups = Boolean.valueOf(parts[3]);
				}
				if (parts.length > 4) {
					useNestedQueries = Boolean.valueOf(parts[4]);
				}
			}
		}
	}
	
	public String getConnectionUrl() {
		return connectionUrl;
	}
	public void setConnectionUrl(String connectionUrl) {
		this.connectionUrl = connectionUrl;
	}
	public String getUsername() {
		return username;
	}
	public void setUsername(String username) {
		this.username = username;
	}
	public String getPassword() {
		return password;
	}
	public void setPassword(String password) {
		this.password = password;
	}
	
	public boolean isValidConfig() {
		return connectionUrl != null && username != null && password != null;
	}
	
	@Override
	public String toString() {
		return "Config url " + getConnectionUrl();
	}

	public boolean isSupportNestedGroups() {
		return supportNestedGroups;
	}

	public void setSupportNestedGroups(boolean supportNestedGroups) {
		this.supportNestedGroups = supportNestedGroups;
	}

	public boolean isUseNestedQueries() {
		return useNestedQueries;
	}
	
	public boolean isUseDirectQueires() {
		return !useNestedQueries;
	}

	public void setUseNestedQueries(boolean useNestedQueries) {
		this.useNestedQueries = useNestedQueries;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((connectionUrl == null) ? 0 : connectionUrl.hashCode());
		result = prime * result + ((password == null) ? 0 : password.hashCode());
		result = prime * result + (supportNestedGroups ? 1231 : 1237);
		result = prime * result + (useNestedQueries ? 1231 : 1237);
		result = prime * result + ((username == null) ? 0 : username.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ConnectionConfig other = (ConnectionConfig) obj;
		if (connectionUrl == null) {
			if (other.connectionUrl != null)
				return false;
		} else if (!connectionUrl.equals(other.connectionUrl))
			return false;
		if (password == null) {
			if (other.password != null)
				return false;
		} else if (!password.equals(other.password))
			return false;
		if (supportNestedGroups != other.supportNestedGroups)
			return false;
		if (useNestedQueries != other.useNestedQueries)
			return false;
		if (username == null) {
			if (other.username != null)
				return false;
		} else if (!username.equals(other.username))
			return false;
		return true;
	}
	
}
