package com.softwareag.example.dirservice.crowd.rest;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;

import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.WebResource.Builder;

public class BaseProvider {
	
	protected static final String ENTITY_TYPE = "entity-type";
	protected static final String START_INDEX = "start-index";
	protected static final String MAX_RESULTS = "max-results";
	protected static final String EXPAND = "expand";
	
	// restriction query
	protected static final String RESTRICTION_PARAM = "restriction";
	protected static final String AND = " and ";
	protected static final String OR  = " or ";
	
	protected Deserializer deserializer = new Deserializer();
	
	protected String executeRequest(WebResource resource, String path, MultivaluedMap<String,String> params) throws Exception {
		return executeRequest(resource, path, params, null);
	}
	
	protected String executeRequest(WebResource resource, String path, MultivaluedMap<String,String> params, String body) throws Exception {
		ClientResponse response;
		Builder builder = resource.path(path).queryParams(params).accept(MediaType.APPLICATION_JSON_TYPE);
		if (isNotEmpty(body)) {
			builder.type(MediaType.APPLICATION_JSON_TYPE);
			response = builder.post(ClientResponse.class, body);
		} else {
			response = builder.get(ClientResponse.class);
		}
		int status = response.getStatus();
		String responseBody = response.getEntity(String.class);
		if (status == 200 || (status >= 400 && status < 500)) {
			return responseBody;
		}
		String errorMessage = "Remote service call error code " + status;
		if (isNotEmpty(responseBody)) {
			errorMessage += ". " + responseBody;
		}
		throw new Exception(errorMessage);
	}	
	
	protected static boolean isEmpty(String value) {
		return value == null || value.trim().isEmpty();
	}
	
	protected static boolean isNotEmpty(String value) {
		return !isEmpty(value);
	}
	
	protected static String like(String value) {
		return "*" + value.trim() + "*";
	}
}
