package com.softwareag.example.dirservice.crowd.rest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.NewCookie;

import com.softwareag.example.dirservice.crowd.ConnectionConfig;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientHandlerException;
import com.sun.jersey.api.client.ClientRequest;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.filter.ClientFilter;
import com.sun.jersey.api.client.filter.HTTPBasicAuthFilter;
import com.webmethods.caf.common.ObjectUtil;

public class CrowdRestConnect {

	private static Map<ConnectionConfig, Client> clientMap = new HashMap<>();
	
	public static Client getClient(ConnectionConfig cfg) {
		Client client = clientMap.get(cfg);
		if (client == null) {
			synchronized (CrowdRestConnect.class) {
				client = configureClient(cfg);
				clientMap.put(cfg, client);
			}
		}
		return client;
	}
	
	public static WebResource getRootResource(ConnectionConfig cfg) {
		Client c = getClient(cfg);
		WebResource w = c.resource(cfg.getConnectionUrl());
		w.accept(MediaType.APPLICATION_JSON_TYPE);
		return w;
	}
	
	public static void clear() {
		synchronized (CrowdRestConnect.class) {
			clientMap.clear();
		}
	}
	
	private static Client configureClient(ConnectionConfig cfg) {
		Client client = Client.create();
		// always use basic auth, session cookie is set to prevent creating multiple sessions
		// Crowd won't work if only the session cookie is present and there is no basic authorization
		client.addFilter( new HTTPBasicAuthFilter(cfg.getUsername(), cfg.getPassword().getBytes()));
		client.addFilter( new CrowdCookieFilter() );
		return client;
	}

	static class CrowdCookieFilter extends ClientFilter {
		private List<Object> cookies;
		
	    @Override
	    public ClientResponse handle(ClientRequest request) throws ClientHandlerException {   	
	        if (cookies != null && !cookies.isEmpty()) {
	            request.getHeaders().put("Cookie", cookies); // put session cookie so session is reused
	        }
	        
	        ClientResponse response = getNext().handle(request);

	        List<NewCookie> receivedCookies = response.getCookies();
	        if (receivedCookies != null && !receivedCookies.isEmpty()) {
	            if (cookies == null) {
	            	cookies = new ArrayList<Object>();
	            }
	            updateCookies(receivedCookies);// simple addAll just for illustration (should probably check for duplicates and expired cookies)
	        }
	        return response;
	    }
	    
	    private void updateCookies(List<NewCookie> receivedCookies) {
	    	for (NewCookie each : receivedCookies) {
	    		addOrReplaceCookie(each);
	    	}
	    }
	    
	    private void addOrReplaceCookie(NewCookie newCookie) {
	    	Iterator<Object> cookieIterator = cookies.iterator();
	    	while (cookieIterator.hasNext()) {
	    		NewCookie cookie = (NewCookie) cookieIterator.next();
	    		if (cookieEquals(cookie,newCookie)) {
	    			cookies.remove(cookie);
	    			cookies.add(newCookie);
	    			return;
	    		}
	    	}
	    	// not found, add cooke
	    	cookies.add(newCookie);
	    }
	    
	    private boolean cookieEquals(NewCookie oldCookie, NewCookie newCookie) {
	    	if (oldCookie.equals(newCookie)) {
	    		return true;
	    	}
	    	if (oldCookie != null && newCookie != null) {
		    	if (
		    		ObjectUtil.equal(oldCookie.getName(), newCookie.getName()) && 
		    		ObjectUtil.equal(oldCookie.getDomain(), newCookie.getDomain()) && 
		    		ObjectUtil.equal(oldCookie.getPath(), newCookie.getPath()) && 
		    		ObjectUtil.equal(oldCookie.getValue(), newCookie.getValue())
		    		) {
		    		return true;
		    	}
	    	}
	    	return false;
	    }

	}
	
	
}


