package com.softwareag.example.dirservice.crowd.rest;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.core.MultivaluedMap;

import com.softwareag.example.dirservice.crowd.CrowdGroupModel;
import com.softwareag.example.dirservice.crowd.CrowdPrincipalModel;
import com.softwareag.example.dirservice.crowd.CrowdUserModel;
import com.softwareag.example.dirservice.crowd.PrincipalListWrapper;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.webmethods.common.search.query.IQueryRefine;
import com.webmethods.common.search.query.ISearchQuery;

public class GroupRestProvider extends BaseProvider {
	private static final String RES_GETGROUP = "/rest/usermanagement/latest/group";
	private static final String RES_SEARCH_GROUPS = "/rest/usermanagement/latest/search";
	private static final String RES_GROUP_MEMBERS_DIRECT = "/rest/usermanagement/latest/group/child-group/direct";
	private static final String RES_PARENT_GROUPS_DIRECT = "/rest/usermanagement/latest/group/parent-group/direct";
	private static final String RES_PARENT_GROUPS_NESTED = "/rest/usermanagement/latest/group/parent-group/nested";
	private static final String RES_USER_MEMBERS_DIRECT = "/rest/usermanagement/latest/group/user/direct";
	
	public CrowdPrincipalModel getGroup(WebResource resource, String groupName) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add("groupname", groupName);
		String s = executeRequest(resource,RES_GETGROUP, params);
		return deserializer.deserialize(s, CrowdGroupModel.class);
	}
	
	public List<CrowdPrincipalModel> getMembership(WebResource resource, boolean parents, String groupid, 
			boolean supportNested, boolean directParents, int resultSize) throws Exception {
		List<CrowdPrincipalModel> result = new ArrayList<CrowdPrincipalModel>();
		if (parents) {
			result.addAll( getParentGroups(resource, groupid, directParents) );
		} else {
			if (supportNested) {
				result.addAll( getMemberGroups(resource, groupid) );
			}
			result.addAll( getMemberUsers(resource, groupid) );
		}
		return result;
	}
	
	private List<CrowdGroupModel> getMemberGroups(WebResource resource, String groupid) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add("groupname", groupid);
		params.add(EXPAND, "group");
		String sGroups = executeRequest(resource, RES_GROUP_MEMBERS_DIRECT, params);
		return deserializer.deserialize(sGroups, PrincipalListWrapper.class).getGroups();
	}
	
	private List<CrowdGroupModel> getParentGroups(WebResource resource, String groupid, boolean directParents) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add("groupname", groupid);
		params.add(EXPAND, "group");
		String sGroups = executeRequest(resource, directParents ? RES_PARENT_GROUPS_DIRECT : RES_PARENT_GROUPS_NESTED, params);
		return deserializer.deserialize(sGroups, PrincipalListWrapper.class).getGroups();
	}
	
	private List<CrowdUserModel> getMemberUsers(WebResource resource, String groupid) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add("groupname", groupid);
		params.add(EXPAND, "user");
		String sGroups = executeRequest(resource, RES_USER_MEMBERS_DIRECT, params);
		return deserializer.deserialize(sGroups, PrincipalListWrapper.class).getUsers();
	}
	
	public List<CrowdGroupModel> searchGroups(WebResource resource, ISearchQuery query, int startIndex, int pageSize) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add(ENTITY_TYPE, "group");
		params.add(EXPAND, "group");
		params.add(START_INDEX, Integer.toString(startIndex));
		params.add(MAX_RESULTS, Integer.toString(pageSize));
		addQueryParams(params, query);
		String s = executeRequest(resource, RES_SEARCH_GROUPS, params);
		return  deserializer.deserialize(s, PrincipalListWrapper.class).getGroups();
	}
	
	private void addQueryParams(MultivaluedMap<String,String> map,  ISearchQuery query) {
		if (query == null || map == null) {
			return;
		}
		IQueryRefine[] refine = query.getRefineQueries();
		String groupName = null;
		if (refine != null) {
			for (IQueryRefine each : refine) {
				if ("group".equals(each.getFieldName())) {
					groupName = each.getValue().trim();
				}
				if ("groupName".equals(each.getFieldName())) {
					if (groupName == null) { // prefer groupid, as we don't have separate name and groupid in Atlassian Crowd
						groupName = each.getValue().trim();
					}
				}
			}

		} else {
			// simple query
			String keyword = query.getKeywordQuery();
			if (isNotEmpty(keyword)) {
				groupName = keyword;
			} 
		}
		
		if (groupName != null) {
			map.add(RESTRICTION_PARAM, "name=" + like(groupName));
		}
	}
}
