package com.softwareag.example.dirservice.crowd.rest;

import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import javax.ws.rs.core.MultivaluedMap;

import org.apache.log4j.Logger;

import com.softwareag.example.dirservice.crowd.CrowdGroupModel;
import com.softwareag.example.dirservice.crowd.CrowdPrincipalModel;
import com.softwareag.example.dirservice.crowd.CrowdUserModel;
import com.softwareag.example.dirservice.crowd.PrincipalListWrapper;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.webmethods.common.search.query.IQueryRefine;
import com.webmethods.common.search.query.ISearchQuery;

public class UserRestProvider extends BaseProvider {
	private static final Logger log = Logger.getLogger(UserRestProvider.class);
	
	private static final String ATTR_USERNAME = "username";
	private static final String RES_AUTHENTICATE = "/rest/usermanagement/latest/authentication";
	private static final String RES_GETUSER = "/rest/usermanagement/latest/user";
	private static final String RES_MEMBERSHIP_DIRECT = "/rest/usermanagement/latest/user/group/direct";
	private static final String RES_MEMBERSHIP_NESTED = "/rest/usermanagement/latest/user/group/nested";
	private static final String RES_SEARCH_USERS = "/rest/usermanagement/latest/search";
	
	/*
	 * Keys that are passed in IQueryRefine from Advanced Search in MWS
	 */
	private static final Set<String> MWS_ADVANCED_QUERY_FIELDS = 
			new LinkedHashSet<>(Arrays.asList( new String[] {"firstName", "lastName", "name", "email"} ));
	
	public boolean authenticate(WebResource resource, String username, String password) {
		boolean result = false;
		StringBuilder sb = new StringBuilder();
		sb.append('{').append("\"value\"").append(':').append('"').append(password).append('"').append('}');
		
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add(ATTR_USERNAME, username);
		try {
			executeRequest(resource, RES_AUTHENTICATE, params, sb.toString());
			result = true;
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return result;
	}
	
	public CrowdPrincipalModel getUser(WebResource resource, String userID) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add(ATTR_USERNAME, userID);
		String s = executeRequest(resource, RES_GETUSER, params);
		return deserializer.deserialize(s, CrowdUserModel.class);
	}
	
	public List<CrowdGroupModel> getMembership(WebResource resource, String userID, boolean directMembership, int resultSize) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add(ATTR_USERNAME, userID);
		if (resultSize > 0) {
			params.add(MAX_RESULTS, Integer.toString(resultSize));
		}
		String s = executeRequest(resource, directMembership ? RES_MEMBERSHIP_DIRECT : RES_MEMBERSHIP_NESTED, params);
		List<CrowdGroupModel> result = deserializer.deserialize(s, PrincipalListWrapper.class).getGroups();
		return result;
	}
	
	public List<CrowdUserModel> searchUsers(WebResource resource, ISearchQuery query, int startIndex, int pageSize) throws Exception {
		MultivaluedMap<String,String> params = new MultivaluedMapImpl();
		params.add(ENTITY_TYPE, "user");
		params.add(EXPAND, "user");
		params.add(START_INDEX, Integer.toString(startIndex));
		params.add(MAX_RESULTS, Integer.toString(pageSize));
		addQueryParams(params, query);
		String s = executeRequest(resource, RES_SEARCH_USERS, params);
		List<CrowdUserModel> result =  deserializer.deserialize(s, PrincipalListWrapper.class).getUsers();
		return result;
	}
	
	private static void addQueryParams(MultivaluedMap<String,String> map,  ISearchQuery query) {
		if (query == null || map == null) {
			return;
		}

		String keywordRestriction = getKeywordRestriction(query.getKeywordQuery());
		String extendedRestriction = getExtendedRestriction(query.getRefineQueries());
		
		String restriction = "";
		if (isNotEmpty(extendedRestriction)) {
			restriction += extendedRestriction;
		}
		if (isNotEmpty(keywordRestriction)) {
			if (isNotEmpty(restriction)) {
				restriction += " AND ";
			}
			restriction += keywordRestriction;
		}
		log.debug("Crowd search query restriction: " + restriction);
		if (isNotEmpty(restriction)) {
			map.add(RESTRICTION_PARAM, restriction);
		}
	}
		
	private static String getKeywordRestriction(String keyword) {
		if (isEmpty(keyword)) {
			return null;
		}
		StringBuilder sb = new StringBuilder();
		sb.append('(');
		sb.append("name=").append(like(keyword)).append(OR);
		sb.append("firstName=").append(like(keyword)).append(OR);
		sb.append("lastName=").append(like(keyword)).append(OR);
		sb.append("email=").append(like(keyword));
		sb.append(')');
		return sb.toString();
	}
	
	private static String getExtendedRestriction(IQueryRefine[] refine) {
		if (refine != null) {
			StringBuilder sb = new StringBuilder();
			for (IQueryRefine each : refine) {
				if (isEmpty(each.getValue())) {
					continue; // skip empty fields
				}
				String key = each.getFieldName();
				if (MWS_ADVANCED_QUERY_FIELDS.contains(key)) {
					sb.append(key).append('=').append(like(each.getValue())).append(AND);
				}				
			}
			String s = sb.toString();
			if (s.endsWith(AND)) {
				s = s.substring(0, s.lastIndexOf(AND));
			}
			if (isNotEmpty(s)) {
				return "(" + s + ")";
			}
		}
		return null;
	}
}
